@@ -7,14 +7,19 @@
 #include "tu_formats.h"
 
 #include "fdl/fd6_format_table.h"
+#include "common/freedreno_ubwc.h"
 
+#include "vk_android.h"
 #include "vk_enum_defines.h"
 #include "vk_util.h"
 #include "drm-uapi/drm_fourcc.h"
 
+#include "tu_android.h"
 #include "tu_device.h"
 #include "tu_image.h"
 
+#include <vulkan/vulkan_android.h>
+
 /* Map non-colorspace-converted YUV formats to RGB pipe formats where we can,
  * since our hardware doesn't support colorspace conversion.
  *
@@ -27,9 +32,6 @@ enum pipe_format
 tu_vk_format_to_pipe_format(VkFormat vk_format)
 {
    switch (vk_format) {
-   case VK_FORMAT_R10X6_UNORM_PACK16:
-   case VK_FORMAT_R10X6G10X6_UNORM_2PACK16:
-      return PIPE_FORMAT_NONE; /* These fail some CTS tests */
    case VK_FORMAT_G8B8G8R8_422_UNORM: /* YUYV */
       return PIPE_FORMAT_R8G8_R8B8_UNORM;
    case VK_FORMAT_B8G8R8G8_422_UNORM: /* UYVY */
@@ -94,109 +96,10 @@ tu6_format_texture(enum pipe_format format, enum a6xx_tile_mode tile_mode)
    return fmt;
 }
 
-enum tu6_ubwc_compat_type {
-   TU6_UBWC_UNKNOWN_COMPAT,
-   TU6_UBWC_R8G8_UNORM,
-   TU6_UBWC_R8G8_INT,
-   TU6_UBWC_R8G8B8A8_UNORM,
-   TU6_UBWC_R8G8B8A8_INT,
-   TU6_UBWC_B8G8R8A8_UNORM,
-   TU6_UBWC_R16G16_UNORM,
-   TU6_UBWC_R16G16_INT,
-   TU6_UBWC_R16G16B16A16_UNORM,
-   TU6_UBWC_R16G16B16A16_INT,
-   TU6_UBWC_R32_INT,
-   TU6_UBWC_R32G32_INT,
-   TU6_UBWC_R32G32B32A32_INT,
-   TU6_UBWC_R32_FLOAT,
-};
-
-static enum tu6_ubwc_compat_type
+static enum fd6_ubwc_compat_type
 tu6_ubwc_compat_mode(const struct fd_dev_info *info, VkFormat format)
 {
-   switch (format) {
-   case VK_FORMAT_R8G8_UNORM:
-   case VK_FORMAT_R8G8_SRGB:
-      return info->a7xx.ubwc_unorm_snorm_int_compatible ?
-         TU6_UBWC_R8G8_INT : TU6_UBWC_R8G8_UNORM;
-
-   case VK_FORMAT_R8G8_SNORM:
-      return info->a7xx.ubwc_unorm_snorm_int_compatible ?
-         TU6_UBWC_R8G8_INT : TU6_UBWC_UNKNOWN_COMPAT;
-
-   case VK_FORMAT_R8G8_UINT:
-   case VK_FORMAT_R8G8_SINT:
-      return TU6_UBWC_R8G8_INT;
-
-   case VK_FORMAT_R8G8B8A8_UNORM:
-   case VK_FORMAT_R8G8B8A8_SRGB:
-   case VK_FORMAT_A8B8G8R8_UNORM_PACK32:
-   case VK_FORMAT_A8B8G8R8_SRGB_PACK32:
-      return info->a7xx.ubwc_unorm_snorm_int_compatible ?
-         TU6_UBWC_R8G8B8A8_INT : TU6_UBWC_R8G8B8A8_UNORM;
-
-   case VK_FORMAT_R8G8B8A8_SNORM:
-      return info->a7xx.ubwc_unorm_snorm_int_compatible ?
-         TU6_UBWC_R8G8B8A8_INT : TU6_UBWC_UNKNOWN_COMPAT;
-
-   case VK_FORMAT_R8G8B8A8_UINT:
-   case VK_FORMAT_R8G8B8A8_SINT:
-   case VK_FORMAT_A8B8G8R8_UINT_PACK32:
-   case VK_FORMAT_A8B8G8R8_SINT_PACK32:
-      return TU6_UBWC_R8G8B8A8_INT;
-
-   case VK_FORMAT_R16G16_UNORM:
-      return info->a7xx.ubwc_unorm_snorm_int_compatible ?
-         TU6_UBWC_R16G16_INT : TU6_UBWC_R16G16_UNORM;
-
-   case VK_FORMAT_R16G16_SNORM:
-      return info->a7xx.ubwc_unorm_snorm_int_compatible ?
-         TU6_UBWC_R16G16_INT : TU6_UBWC_UNKNOWN_COMPAT;
-
-   case VK_FORMAT_R16G16_UINT:
-   case VK_FORMAT_R16G16_SINT:
-      return TU6_UBWC_R16G16_INT;
-
-   case VK_FORMAT_R16G16B16A16_UNORM:
-      return info->a7xx.ubwc_unorm_snorm_int_compatible ?
-         TU6_UBWC_R16G16B16A16_INT : TU6_UBWC_R16G16B16A16_UNORM;
-
-   case VK_FORMAT_R16G16B16A16_SNORM:
-      return info->a7xx.ubwc_unorm_snorm_int_compatible ?
-         TU6_UBWC_R16G16B16A16_INT : TU6_UBWC_UNKNOWN_COMPAT;
-
-   case VK_FORMAT_R16G16B16A16_UINT:
-   case VK_FORMAT_R16G16B16A16_SINT:
-      return TU6_UBWC_R16G16B16A16_INT;
-
-   case VK_FORMAT_R32_UINT:
-   case VK_FORMAT_R32_SINT:
-      return TU6_UBWC_R32_INT;
-
-   case VK_FORMAT_R32G32_UINT:
-   case VK_FORMAT_R32G32_SINT:
-      return TU6_UBWC_R32G32_INT;
-
-   case VK_FORMAT_R32G32B32A32_UINT:
-   case VK_FORMAT_R32G32B32A32_SINT:
-      return TU6_UBWC_R32G32B32A32_INT;
-
-   case VK_FORMAT_D32_SFLOAT:
-   case VK_FORMAT_R32_SFLOAT:
-      /* TODO: a630 blob allows these, but not a660.  When is it legal? */
-      return TU6_UBWC_UNKNOWN_COMPAT;
-
-   case VK_FORMAT_B8G8R8A8_UNORM:
-   case VK_FORMAT_B8G8R8A8_SRGB:
-      /* The blob doesn't list these as compatible, but they surely are.
-       * freedreno's happy to cast between them, and zink would really like
-       * to.
-       */
-      return TU6_UBWC_B8G8R8A8_UNORM;
-
-   default:
-      return TU6_UBWC_UNKNOWN_COMPAT;
-   }
+   return fd6_ubwc_compat_mode(info, vk_format_to_pipe_format(format));
 }
 
 bool
@@ -212,9 +115,9 @@ tu6_mutable_format_list_ubwc_compatible(const struct fd_dev_info *info,
    if (fmt_list->viewFormatCount == 1)
       return true;
 
-   enum tu6_ubwc_compat_type type =
+   enum fd6_ubwc_compat_type type =
       tu6_ubwc_compat_mode(info, fmt_list->pViewFormats[0]);
-   if (type == TU6_UBWC_UNKNOWN_COMPAT)
+   if (type == FD6_UBWC_UNKNOWN_COMPAT)
       return false;
 
    for (uint32_t i = 1; i < fmt_list->viewFormatCount; i++) {
@@ -630,6 +533,14 @@ tu_get_image_format_properties(
       }
    }
 
+   if (image_usage & VK_IMAGE_USAGE_INPUT_ATTACHMENT_BIT) {
+      if (!(format_feature_flags &
+            (VK_FORMAT_FEATURE_COLOR_ATTACHMENT_BIT |
+             VK_FORMAT_FEATURE_DEPTH_STENCIL_ATTACHMENT_BIT))) {
+         return tu_image_unsupported_format(pImageFormatProperties);
+      }
+   }
+
    *pImageFormatProperties = (VkImageFormatProperties) {
       .maxExtent = maxExtent,
       .maxMipLevels = maxMipLevels,
@@ -685,6 +596,14 @@ tu_get_external_image_format_properties(
                           handleType, pImageFormatInfo->type);
       }
       break;
+#if DETECT_OS_ANDROID
+   case VK_EXTERNAL_MEMORY_HANDLE_TYPE_ANDROID_HARDWARE_BUFFER_BIT_ANDROID:
+      flags = VK_EXTERNAL_MEMORY_FEATURE_DEDICATED_ONLY_BIT |
+              VK_EXTERNAL_MEMORY_FEATURE_EXPORTABLE_BIT |
+              VK_EXTERNAL_MEMORY_FEATURE_IMPORTABLE_BIT;
+      compat_flags = export_flags = VK_EXTERNAL_MEMORY_HANDLE_TYPE_ANDROID_HARDWARE_BUFFER_BIT_ANDROID;
+      break;
+#endif
    case VK_EXTERNAL_MEMORY_HANDLE_TYPE_HOST_ALLOCATION_BIT_EXT:
       flags = VK_EXTERNAL_MEMORY_FEATURE_IMPORTABLE_BIT;
       compat_flags = VK_EXTERNAL_MEMORY_HANDLE_TYPE_HOST_ALLOCATION_BIT_EXT;
@@ -717,6 +636,7 @@ tu_GetPhysicalDeviceImageFormatProperties2(
    const VkPhysicalDeviceExternalImageFormatInfo *external_info = NULL;
    const VkPhysicalDeviceImageViewImageFormatInfoEXT *image_view_info = NULL;
    VkExternalImageFormatProperties *external_props = NULL;
+   VkAndroidHardwareBufferUsageANDROID *android_usage = NULL;
    VkFilterCubicImageViewImageFormatPropertiesEXT *cubic_props = NULL;
    VkFormatFeatureFlags format_feature_flags;
    VkSamplerYcbcrConversionImageFormatProperties *ycbcr_props = NULL;
@@ -749,6 +669,9 @@ tu_GetPhysicalDeviceImageFormatProperties2(
       case VK_STRUCTURE_TYPE_EXTERNAL_IMAGE_FORMAT_PROPERTIES:
          external_props = (VkExternalImageFormatProperties *) s;
          break;
+      case VK_STRUCTURE_TYPE_ANDROID_HARDWARE_BUFFER_USAGE_ANDROID:
+         android_usage = (VkAndroidHardwareBufferUsageANDROID *) s;
+         break;
       case VK_STRUCTURE_TYPE_FILTER_CUBIC_IMAGE_VIEW_IMAGE_FORMAT_PROPERTIES_EXT:
          cubic_props = (VkFilterCubicImageViewImageFormatPropertiesEXT *) s;
          break;
@@ -789,6 +712,43 @@ tu_GetPhysicalDeviceImageFormatProperties2(
       }
    }
 
+   if (android_usage) {
+      /* Don't expect gralloc to be able to allocate anything other than 3D: */
+      if (base_info->type != VK_IMAGE_TYPE_2D) {
+         result = vk_errorf(physical_device, VK_ERROR_FORMAT_NOT_SUPPORTED,
+                            "type (%u) unsupported for AHB", base_info->type);
+         goto fail;
+      }
+      VkImageFormatProperties *props = &base_props->imageFormatProperties;
+      if (!(props->sampleCounts & VK_SAMPLE_COUNT_1_BIT)) {
+         result = vk_errorf(physical_device, VK_ERROR_FORMAT_NOT_SUPPORTED,
+                          "sampleCounts (%x) unsupported for AHB", props->sampleCounts);
+         goto fail;
+      }
+      android_usage->androidHardwareBufferUsage =
+         vk_image_usage_to_ahb_usage(base_info->flags, base_info->usage);
+      uint32_t format = vk_image_format_to_ahb_format(base_info->format);
+      if (!format) {
+         result = vk_errorf(physical_device, VK_ERROR_FORMAT_NOT_SUPPORTED,
+                            "format (%u) unsupported for AHB", base_info->format);
+         goto fail;
+      }
+      /* We can't advertise support for anything that gralloc cannot allocate
+       * so we are stuck without any better option than attempting a test
+       * allocation:
+       */
+      if (!vk_ahb_probe_format(base_info->format, base_info->flags, base_info->usage)) {
+         result = vk_errorf(physical_device, VK_ERROR_FORMAT_NOT_SUPPORTED,
+                            "format (%x) with flags (%x) and usage (%x) unsupported for AHB",
+                            base_info->format, base_info->flags, base_info->usage);
+         goto fail;
+      }
+
+      /* AHBs with mipmap usage will ignore this property */
+      props->maxMipLevels = 1;
+      props->sampleCounts = VK_SAMPLE_COUNT_1_BIT;
+   }
+
    if (ycbcr_props)
       ycbcr_props->combinedImageSamplerDescriptorCount = 1;
 
@@ -819,36 +779,3 @@ tu_GetPhysicalDeviceSparseImageFormatProperties2(
    /* Sparse images are not yet supported. */
    *pPropertyCount = 0;
 }
-
-VKAPI_ATTR void VKAPI_CALL
-tu_GetPhysicalDeviceExternalBufferProperties(
-   VkPhysicalDevice physicalDevice,
-   const VkPhysicalDeviceExternalBufferInfo *pExternalBufferInfo,
-   VkExternalBufferProperties *pExternalBufferProperties)
-{
-   BITMASK_ENUM(VkExternalMemoryFeatureFlagBits) flags = 0;
-   VkExternalMemoryHandleTypeFlags export_flags = 0;
-   VkExternalMemoryHandleTypeFlags compat_flags = 0;
-   switch (pExternalBufferInfo->handleType) {
-   case VK_EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_FD_BIT:
-   case VK_EXTERNAL_MEMORY_HANDLE_TYPE_DMA_BUF_BIT_EXT:
-      flags = VK_EXTERNAL_MEMORY_FEATURE_EXPORTABLE_BIT |
-              VK_EXTERNAL_MEMORY_FEATURE_IMPORTABLE_BIT;
-      compat_flags = export_flags =
-         VK_EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_FD_BIT |
-         VK_EXTERNAL_MEMORY_HANDLE_TYPE_DMA_BUF_BIT_EXT;
-      break;
-   case VK_EXTERNAL_MEMORY_HANDLE_TYPE_HOST_ALLOCATION_BIT_EXT:
-      flags = VK_EXTERNAL_MEMORY_FEATURE_IMPORTABLE_BIT;
-      compat_flags = VK_EXTERNAL_MEMORY_HANDLE_TYPE_HOST_ALLOCATION_BIT_EXT;
-      break;
-   default:
-      break;
-   }
-   pExternalBufferProperties->externalMemoryProperties =
-      (VkExternalMemoryProperties) {
-         .externalMemoryFeatures = flags,
-         .exportFromImportedHandleTypes = export_flags,
-         .compatibleHandleTypes = compat_flags,
-      };
-}
