@@ -275,6 +275,7 @@ get_program_state(struct fd_context *ctx, const struct pipe_draw_info *info)
    return fd6_ctx->prog;
 }
 
+template <chip CHIP>
 static void
 flush_streamout(struct fd_context *ctx, struct fd6_emit *emit)
    assert_dt
@@ -286,8 +287,8 @@ flush_streamout(struct fd_context *ctx, struct fd6_emit *emit)
 
    for (unsigned i = 0; i < PIPE_MAX_SO_BUFFERS; i++) {
       if (emit->streamout_mask & (1 << i)) {
-         enum vgt_event_type evt = (enum vgt_event_type)(FLUSH_SO_0 + i);
-         fd6_event_write(ctx->batch, ring, evt, false);
+         enum fd_gpu_event evt = (enum fd_gpu_event)(FD_FLUSH_SO_0 + i);
+         fd6_event_write<CHIP>(ctx, ring, evt);
       }
    }
 }
@@ -463,7 +464,7 @@ draw_vbos(struct fd_context *ctx, const struct pipe_draw_info *info,
       ctx->batch->barrier |= FD6_WAIT_FOR_ME;
 
    if (ctx->batch->barrier)
-      fd6_barrier_flush(ctx->batch);
+      fd6_barrier_flush<CHIP>(ctx->batch);
 
    /* for debug after a lock up, write a unique counter value
     * to scratch7 for each draw, to make it easier to match up
@@ -507,7 +508,7 @@ draw_vbos(struct fd_context *ctx, const struct pipe_draw_info *info,
          uint32_t last_index_start = ctx->last.index_start;
 
          for (unsigned i = 1; i < num_draws; i++) {
-            flush_streamout(ctx, &emit);
+            flush_streamout<CHIP>(ctx, &emit);
 
             fd6_vsc_update_sizes(ctx->batch, info, &draws[i]);
 
@@ -536,7 +537,7 @@ draw_vbos(struct fd_context *ctx, const struct pipe_draw_info *info,
 
    emit_marker6(ring, 7);
 
-   flush_streamout(ctx, &emit);
+   flush_streamout<CHIP>(ctx, &emit);
 
    fd_context_all_clean(ctx);
 }
@@ -687,7 +688,4 @@ fd6_draw_init(struct pipe_context *pctx)
    ctx->update_draw = fd6_update_draw<CHIP>;
    fd6_update_draw<CHIP>(ctx);
 }
-
-/* Teach the compiler about needed variants: */
-template void fd6_draw_init<A6XX>(struct pipe_context *pctx);
-template void fd6_draw_init<A7XX>(struct pipe_context *pctx);
+FD_GENX(fd6_draw_init);
