@@ -26,8 +26,8 @@
  */
 
 #include "v3d_context.h"
+#include "v3d_screen.h"
 
-#include "compiler/nir/nir_serialize.h"
 #include "util/blob.h"
 #include "util/u_upload_mgr.h"
 
@@ -52,14 +52,6 @@ v3d_key_size(gl_shader_stage stage)
 
 void v3d_disk_cache_init(struct v3d_screen *screen)
 {
-        char *renderer;
-
-        ASSERTED int len =
-                asprintf(&renderer, "V3D %d.%d",
-                         screen->devinfo.ver / 10,
-                         screen->devinfo.ver % 10);
-        assert(len > 0);
-
         const struct build_id_note *note =
                 build_id_find_nhdr_for_addr(v3d_disk_cache_init);
         assert(note && build_id_length(note) == 20);
@@ -70,33 +62,30 @@ void v3d_disk_cache_init(struct v3d_screen *screen)
         char timestamp[41];
         _mesa_sha1_format(timestamp, id_sha1);
 
-        screen->disk_cache = disk_cache_create(renderer, timestamp, v3d_mesa_debug);
-
-        free(renderer);
+        screen->disk_cache =
+                disk_cache_create(v3d_screen_get_name((struct pipe_screen *) screen),
+                                  timestamp, v3d_mesa_debug);
 }
 
 static void
 v3d_disk_cache_compute_key(struct disk_cache *cache,
                            const struct v3d_key *key,
-                           cache_key cache_key)
+                           cache_key cache_key,
+                           const struct v3d_uncompiled_shader *uncompiled)
 {
         assert(cache);
 
-        struct v3d_uncompiled_shader *uncompiled = key->shader_state;
         assert(uncompiled->base.type == PIPE_SHADER_IR_NIR);
         nir_shader *nir = uncompiled->base.ir.nir;
 
-        struct blob blob;
-        blob_init(&blob);
-
         uint32_t ckey_size = v3d_key_size(nir->info.stage);
         struct v3d_key *ckey = malloc(ckey_size);
         memcpy(ckey, key, ckey_size);
-        ckey->shader_state = NULL;
 
+        struct blob blob;
+        blob_init(&blob);
         blob_write_bytes(&blob, ckey, ckey_size);
-
-        nir_serialize(&blob, nir, true);
+        blob_write_bytes(&blob, uncompiled->sha1, 20);
 
         disk_cache_compute_key(cache, blob.data, blob.size, cache_key);
 
@@ -106,7 +95,8 @@ v3d_disk_cache_compute_key(struct disk_cache *cache,
 
 struct v3d_compiled_shader *
 v3d_disk_cache_retrieve(struct v3d_context *v3d,
-                        const struct v3d_key *key)
+                        const struct v3d_key *key,
+                        const struct v3d_uncompiled_shader *uncompiled)
 {
         struct v3d_screen *screen = v3d->screen;
         struct disk_cache *cache = screen->disk_cache;
@@ -114,12 +104,11 @@ v3d_disk_cache_retrieve(struct v3d_context *v3d,
         if (!cache)
                 return NULL;
 
-        struct v3d_uncompiled_shader *uncompiled = key->shader_state;
         assert(uncompiled->base.type == PIPE_SHADER_IR_NIR);
         nir_shader *nir = uncompiled->base.ir.nir;
 
         cache_key cache_key;
-        v3d_disk_cache_compute_key(cache, key, cache_key);
+        v3d_disk_cache_compute_key(cache, key, cache_key, uncompiled);
 
         size_t buffer_size;
         void *buffer = disk_cache_get(cache, cache_key, &buffer_size);
@@ -188,6 +177,7 @@ v3d_disk_cache_retrieve(struct v3d_context *v3d,
 void
 v3d_disk_cache_store(struct v3d_context *v3d,
                      const struct v3d_key *key,
+                     const struct v3d_uncompiled_shader *uncompiled,
                      const struct v3d_compiled_shader *shader,
                      uint64_t *qpu_insts,
                      uint32_t qpu_size)
@@ -198,12 +188,11 @@ v3d_disk_cache_store(struct v3d_context *v3d,
         if (!cache)
                 return;
 
-        struct v3d_uncompiled_shader *uncompiled = key->shader_state;
         assert(uncompiled->base.type == PIPE_SHADER_IR_NIR);
         nir_shader *nir = uncompiled->base.ir.nir;
 
         cache_key cache_key;
-        v3d_disk_cache_compute_key(cache, key, cache_key);
+        v3d_disk_cache_compute_key(cache, key, cache_key, uncompiled);
 
         if (V3D_DBG(CACHE)) {
                 char sha1[41];
