@@ -2,6 +2,7 @@ use crate::api::icd::*;
 use crate::core::device::*;
 use crate::core::event::*;
 use crate::core::memory::*;
+use crate::core::platform::*;
 use crate::core::program::*;
 use crate::core::queue::*;
 use crate::impl_cl_type_trait;
@@ -10,7 +11,6 @@ use mesa_rust::compiler::clc::*;
 use mesa_rust::compiler::nir::*;
 use mesa_rust::nir_pass;
 use mesa_rust::pipe::context::RWFlags;
-use mesa_rust::pipe::context::ResourceMapType;
 use mesa_rust::pipe::resource::*;
 use mesa_rust::pipe::screen::ResourceType;
 use mesa_rust_gen::*;
@@ -21,6 +21,7 @@ use rusticl_opencl_gen::*;
 use std::cmp;
 use std::collections::HashMap;
 use std::convert::TryInto;
+use std::mem::size_of;
 use std::os::raw::c_void;
 use std::ptr;
 use std::slice;
@@ -60,6 +61,8 @@ pub enum InternalKernelArgType {
     FormatArray,
     OrderArray,
     WorkDim,
+    WorkGroupOffsets,
+    NumWorkgroups,
 }
 
 #[derive(Hash, PartialEq, Eq, Clone)]
@@ -220,6 +223,8 @@ impl InternalKernelArg {
             InternalKernelArgType::FormatArray => bin.push(4),
             InternalKernelArgType::OrderArray => bin.push(5),
             InternalKernelArgType::WorkDim => bin.push(6),
+            InternalKernelArgType::WorkGroupOffsets => bin.push(7),
+            InternalKernelArgType::NumWorkgroups => bin.push(8),
         }
 
         bin
@@ -242,6 +247,8 @@ impl InternalKernelArg {
             4 => InternalKernelArgType::FormatArray,
             5 => InternalKernelArgType::OrderArray,
             6 => InternalKernelArgType::WorkDim,
+            7 => InternalKernelArgType::WorkGroupOffsets,
+            8 => InternalKernelArgType::NumWorkgroups,
             _ => return None,
         };
 
@@ -302,21 +309,22 @@ pub struct Kernel {
     pub name: String,
     values: Mutex<Vec<Option<KernelArgValue>>>,
     builds: HashMap<&'static Device, Arc<NirKernelBuild>>,
-    pub kernel_info: KernelInfo,
+    pub kernel_info: Arc<KernelInfo>,
 }
 
 impl_cl_type_trait!(cl_kernel, Kernel, CL_INVALID_KERNEL);
 
-fn create_kernel_arr<T>(vals: &[usize], val: T) -> [T; 3]
+fn create_kernel_arr<T>(vals: &[usize], val: T) -> CLResult<[T; 3]>
 where
     T: std::convert::TryFrom<usize> + Copy,
     <T as std::convert::TryFrom<usize>>::Error: std::fmt::Debug,
 {
     let mut res = [val; 3];
     for (i, v) in vals.iter().enumerate() {
-        res[i] = (*v).try_into().expect("64 bit work groups not supported");
+        res[i] = (*v).try_into().ok().ok_or(CL_OUT_OF_RESOURCES)?;
     }
-    res
+
+    Ok(res)
 }
 
 fn opt_nir(nir: &mut NirShader, dev: &Device, has_explicit_types: bool) {
@@ -401,19 +409,19 @@ fn lower_and_optimize_nir(
     args: &[spirv::SPIRVKernelArg],
     lib_clc: &NirShader,
 ) -> (Vec<KernelArg>, Vec<InternalKernelArg>) {
-    let address_bits_base_type;
     let address_bits_ptr_type;
+    let address_bits_base_type;
     let global_address_format;
     let shared_address_format;
 
     if dev.address_bits() == 64 {
-        address_bits_base_type = glsl_base_type::GLSL_TYPE_UINT64;
         address_bits_ptr_type = unsafe { glsl_uint64_t_type() };
+        address_bits_base_type = glsl_base_type::GLSL_TYPE_UINT64;
         global_address_format = nir_address_format::nir_address_format_64bit_global;
         shared_address_format = nir_address_format::nir_address_format_32bit_offset_as_64bit;
     } else {
-        address_bits_base_type = glsl_base_type::GLSL_TYPE_UINT;
         address_bits_ptr_type = unsafe { glsl_uint_type() };
+        address_bits_base_type = glsl_base_type::GLSL_TYPE_UINT;
         global_address_format = nir_address_format::nir_address_format_32bit_global;
         shared_address_format = nir_address_format::nir_address_format_32bit_offset;
     }
@@ -451,6 +459,8 @@ fn lower_and_optimize_nir(
     nir_pass!(nir, nir_dedup_inline_samplers);
 
     let printf_opts = nir_lower_printf_options {
+        ptr_bit_size: 0,
+        use_printf_base_identifier: false,
         max_buffer_size: dev.printf_buffer_size() as u32,
     };
     nir_pass!(nir, nir_lower_printf, &printf_opts);
@@ -526,6 +536,7 @@ fn lower_and_optimize_nir(
     nir_pass!(nir, nir_lower_system_values);
     let mut compute_options = nir_lower_compute_system_values_options::default();
     compute_options.set_has_base_global_invocation_id(true);
+    compute_options.set_has_base_workgroup_id(true);
     nir_pass!(nir, nir_lower_compute_system_values, &compute_options);
     nir.gather_info();
 
@@ -533,7 +544,7 @@ fn lower_and_optimize_nir(
         internal_args.push(InternalKernelArg {
             kind: InternalKernelArgType::GlobalWorkOffsets,
             offset: 0,
-            size: (3 * dev.address_bits() / 8) as usize,
+            size: 3 * size_of::<usize>(),
         });
         lower_state.base_global_invoc_id_loc = args.len() + internal_args.len() - 1;
         nir.add_var(
@@ -544,6 +555,37 @@ fn lower_and_optimize_nir(
         );
     }
 
+    if nir.reads_sysval(gl_system_value::SYSTEM_VALUE_BASE_WORKGROUP_ID) {
+        internal_args.push(InternalKernelArg {
+            kind: InternalKernelArgType::WorkGroupOffsets,
+            offset: 0,
+            size: 3 * size_of::<usize>(),
+        });
+        lower_state.base_workgroup_id_loc = args.len() + internal_args.len() - 1;
+        nir.add_var(
+            nir_variable_mode::nir_var_uniform,
+            unsafe { glsl_vector_type(address_bits_base_type, 3) },
+            lower_state.base_workgroup_id_loc,
+            "base_workgroup_id",
+        );
+    }
+
+    if nir.reads_sysval(gl_system_value::SYSTEM_VALUE_NUM_WORKGROUPS) {
+        internal_args.push(InternalKernelArg {
+            kind: InternalKernelArgType::NumWorkgroups,
+            offset: 0,
+            size: 12,
+        });
+
+        lower_state.num_workgroups_loc = args.len() + internal_args.len() - 1;
+        nir.add_var(
+            nir_variable_mode::nir_var_uniform,
+            unsafe { glsl_vector_type(glsl_base_type::GLSL_TYPE_UINT, 3) },
+            lower_state.num_workgroups_loc,
+            "num_workgroups",
+        );
+    }
+
     if nir.has_constant() {
         internal_args.push(InternalKernelArg {
             kind: InternalKernelArgType::ConstantBuffer,
@@ -811,9 +853,8 @@ fn extract<'a, const S: usize>(buf: &'a mut &[u8]) -> &'a [u8; S] {
 }
 
 impl Kernel {
-    pub fn new(name: String, prog: Arc<Program>) -> Arc<Kernel> {
-        let prog_build = prog.build_info();
-        let kernel_info = prog_build.kernel_info.get(&name).unwrap().clone();
+    pub fn new(name: String, prog: Arc<Program>, prog_build: &ProgramBuild) -> Arc<Kernel> {
+        let kernel_info = Arc::clone(prog_build.kernel_info.get(&name).unwrap());
         let builds = prog_build
             .builds
             .iter()
@@ -823,7 +864,7 @@ impl Kernel {
         let values = vec![None; kernel_info.args.len()];
         Arc::new(Self {
             base: CLObjectBase::new(RusticlTypes::Kernel),
-            prog: prog.clone(),
+            prog: prog,
             name: name,
             values: Mutex::new(values),
             builds: builds,
@@ -867,27 +908,23 @@ impl Kernel {
         }
     }
 
-    fn optimize_local_size(&self, d: &Device, grid: &mut [u32; 3], block: &mut [u32; 3]) {
+    fn optimize_local_size(&self, d: &Device, grid: &mut [usize; 3], block: &mut [u32; 3]) {
         if !block.contains(&0) {
             for i in 0..3 {
                 // we already made sure everything is fine
-                grid[i] /= block[i];
+                grid[i] /= block[i] as usize;
             }
             return;
         }
 
-        let mut usize_grid = [0usize; 3];
         let mut usize_block = [0usize; 3];
-
         for i in 0..3 {
-            usize_grid[i] = grid[i] as usize;
             usize_block[i] = block[i] as usize;
         }
 
-        self.suggest_local_size(d, 3, &mut usize_grid, &mut usize_block);
+        self.suggest_local_size(d, 3, grid, &mut usize_block);
 
         for i in 0..3 {
-            grid[i] = usize_grid[i] as u32;
             block[i] = usize_block[i] as u32;
         }
     }
@@ -902,62 +939,87 @@ impl Kernel {
         grid: &[usize],
         offsets: &[usize],
     ) -> CLResult<EventSig> {
-        let nir_kernel_build = self.builds.get(q.device).unwrap().clone();
-        let mut block = create_kernel_arr::<u32>(block, 1);
-        let mut grid = create_kernel_arr::<u32>(grid, 1);
-        let offsets = create_kernel_arr::<u64>(offsets, 0);
-        let mut input: Vec<u8> = Vec::new();
-        let mut resource_info = Vec::new();
-        // Set it once so we get the alignment padding right
-        let static_local_size: u64 = nir_kernel_build.shared_size;
-        let mut variable_local_size: u64 = static_local_size;
-        let printf_size = q.device.printf_buffer_size() as u32;
-        let mut samplers = Vec::new();
-        let mut iviews = Vec::new();
-        let mut sviews = Vec::new();
-        let mut tex_formats: Vec<u16> = Vec::new();
-        let mut tex_orders: Vec<u16> = Vec::new();
-        let mut img_formats: Vec<u16> = Vec::new();
-        let mut img_orders: Vec<u16> = Vec::new();
-        let null_ptr: &[u8] = if q.device.address_bits() == 64 {
-            &[0; 8]
-        } else {
-            &[0; 4]
-        };
+        // Clone all the data we need to execute this kernel
+        let kernel_info = Arc::clone(&self.kernel_info);
+        let arg_values = self.arg_values().clone();
+        let nir_kernel_build = Arc::clone(&self.builds[q.device]);
+
+        // operations we want to report errors to the clients
+        let mut block = create_kernel_arr::<u32>(block, 1)?;
+        let mut grid = create_kernel_arr::<usize>(grid, 1)?;
+        let offsets = create_kernel_arr::<usize>(offsets, 0)?;
 
         self.optimize_local_size(q.device, &mut grid, &mut block);
 
-        let arg_values = self.arg_values();
-        for (arg, val) in self.kernel_info.args.iter().zip(arg_values.iter()) {
-            if arg.dead {
-                continue;
-            }
+        Ok(Box::new(move |q, ctx| {
+            let mut workgroup_id_offset_loc = None;
+            let mut input = Vec::new();
+            // Set it once so we get the alignment padding right
+            let static_local_size: u64 = nir_kernel_build.shared_size;
+            let mut variable_local_size: u64 = static_local_size;
+            let printf_size = q.device.printf_buffer_size() as u32;
+            let mut samplers = Vec::new();
+            let mut iviews = Vec::new();
+            let mut sviews = Vec::new();
+            let mut tex_formats: Vec<u16> = Vec::new();
+            let mut tex_orders: Vec<u16> = Vec::new();
+            let mut img_formats: Vec<u16> = Vec::new();
+            let mut img_orders: Vec<u16> = Vec::new();
 
-            if arg.kind != KernelArgType::Image
-                && arg.kind != KernelArgType::RWImage
-                && arg.kind != KernelArgType::Texture
-                && arg.kind != KernelArgType::Sampler
-            {
-                input.resize(arg.offset, 0);
-            }
-            match val.as_ref().unwrap() {
-                KernelArgValue::Constant(c) => input.extend_from_slice(c),
-                KernelArgValue::Buffer(buffer) => {
-                    let res = buffer.get_res_of_dev(q.device)?;
-                    if q.device.address_bits() == 64 {
-                        input.extend_from_slice(&buffer.offset.to_ne_bytes());
-                    } else {
-                        input.extend_from_slice(&(buffer.offset as u32).to_ne_bytes());
-                    }
-                    resource_info.push((res.clone(), arg.offset));
+            let null_ptr;
+            let null_ptr_v3;
+            if q.device.address_bits() == 64 {
+                null_ptr = [0u8; 8].as_slice();
+                null_ptr_v3 = [0u8; 24].as_slice();
+            } else {
+                null_ptr = [0u8; 4].as_slice();
+                null_ptr_v3 = [0u8; 12].as_slice();
+            };
+
+            let mut resource_info = Vec::new();
+            fn add_global<'a>(
+                q: &Queue,
+                input: &mut Vec<u8>,
+                resource_info: &mut Vec<(&'a PipeResource, usize)>,
+                res: &'a PipeResource,
+                offset: usize,
+            ) {
+                resource_info.push((res, input.len()));
+                if q.device.address_bits() == 64 {
+                    let offset: u64 = offset as u64;
+                    input.extend_from_slice(&offset.to_ne_bytes());
+                } else {
+                    let offset: u32 = offset as u32;
+                    input.extend_from_slice(&offset.to_ne_bytes());
                 }
-                KernelArgValue::Image(image) => {
-                    let res = image.get_res_of_dev(q.device)?;
+            }
 
-                    // If resource is a buffer, the image was created from a buffer. Use strides and
-                    // dimensions of the image then.
-                    let app_img_info =
-                        if res.as_ref().is_buffer() && image.mem_type == CL_MEM_OBJECT_IMAGE2D {
+            for (arg, val) in kernel_info.args.iter().zip(arg_values.iter()) {
+                if arg.dead {
+                    continue;
+                }
+
+                if arg.kind != KernelArgType::Image
+                    && arg.kind != KernelArgType::RWImage
+                    && arg.kind != KernelArgType::Texture
+                    && arg.kind != KernelArgType::Sampler
+                {
+                    input.resize(arg.offset, 0);
+                }
+                match val.as_ref().unwrap() {
+                    KernelArgValue::Constant(c) => input.extend_from_slice(c),
+                    KernelArgValue::Buffer(buffer) => {
+                        let res = buffer.get_res_of_dev(q.device)?;
+                        add_global(q, &mut input, &mut resource_info, res, buffer.offset);
+                    }
+                    KernelArgValue::Image(image) => {
+                        let res = image.get_res_of_dev(q.device)?;
+
+                        // If resource is a buffer, the image was created from a buffer. Use strides and
+                        // dimensions of the image then.
+                        let app_img_info = if res.as_ref().is_buffer()
+                            && image.mem_type == CL_MEM_OBJECT_IMAGE2D
+                        {
                             Some(AppImgInfo::new(
                                 image.image_desc.row_pitch()? / image.image_elem_size as u32,
                                 image.image_desc.width()?,
@@ -967,131 +1029,140 @@ impl Kernel {
                             None
                         };
 
-                    let format = image.pipe_format;
-                    let (formats, orders) = if arg.kind == KernelArgType::Image {
-                        iviews.push(res.pipe_image_view(
-                            format,
-                            false,
-                            image.pipe_image_host_access(),
-                            app_img_info.as_ref(),
-                        ));
-                        (&mut img_formats, &mut img_orders)
-                    } else if arg.kind == KernelArgType::RWImage {
-                        iviews.push(res.pipe_image_view(
-                            format,
-                            true,
-                            image.pipe_image_host_access(),
-                            app_img_info.as_ref(),
-                        ));
-                        (&mut img_formats, &mut img_orders)
-                    } else {
-                        sviews.push((res.clone(), format, app_img_info));
-                        (&mut tex_formats, &mut tex_orders)
-                    };
+                        let format = image.pipe_format;
+                        let (formats, orders) = if arg.kind == KernelArgType::Image {
+                            iviews.push(res.pipe_image_view(
+                                format,
+                                false,
+                                image.pipe_image_host_access(),
+                                app_img_info.as_ref(),
+                            ));
+                            (&mut img_formats, &mut img_orders)
+                        } else if arg.kind == KernelArgType::RWImage {
+                            iviews.push(res.pipe_image_view(
+                                format,
+                                true,
+                                image.pipe_image_host_access(),
+                                app_img_info.as_ref(),
+                            ));
+                            (&mut img_formats, &mut img_orders)
+                        } else {
+                            sviews.push((res.clone(), format, app_img_info));
+                            (&mut tex_formats, &mut tex_orders)
+                        };
 
-                    let binding = arg.binding as usize;
-                    assert!(binding >= formats.len());
+                        let binding = arg.binding as usize;
+                        assert!(binding >= formats.len());
 
-                    formats.resize(binding, 0);
-                    orders.resize(binding, 0);
+                        formats.resize(binding, 0);
+                        orders.resize(binding, 0);
 
-                    formats.push(image.image_format.image_channel_data_type as u16);
-                    orders.push(image.image_format.image_channel_order as u16);
-                }
-                KernelArgValue::LocalMem(size) => {
-                    // TODO 32 bit
-                    let pot = cmp::min(*size, 0x80);
-                    variable_local_size =
-                        align(variable_local_size, pot.next_power_of_two() as u64);
-                    if q.device.address_bits() == 64 {
-                        input.extend_from_slice(&variable_local_size.to_ne_bytes());
-                    } else {
-                        input.extend_from_slice(&(variable_local_size as u32).to_ne_bytes());
+                        formats.push(image.image_format.image_channel_data_type as u16);
+                        orders.push(image.image_format.image_channel_order as u16);
+                    }
+                    KernelArgValue::LocalMem(size) => {
+                        // TODO 32 bit
+                        let pot = cmp::min(*size, 0x80);
+                        variable_local_size =
+                            align(variable_local_size, pot.next_power_of_two() as u64);
+                        if q.device.address_bits() == 64 {
+                            let variable_local_size: [u8; 8] = variable_local_size.to_ne_bytes();
+                            input.extend_from_slice(&variable_local_size);
+                        } else {
+                            let variable_local_size: [u8; 4] =
+                                (variable_local_size as u32).to_ne_bytes();
+                            input.extend_from_slice(&variable_local_size);
+                        }
+                        variable_local_size += *size as u64;
+                    }
+                    KernelArgValue::Sampler(sampler) => {
+                        samplers.push(sampler.pipe());
+                    }
+                    KernelArgValue::None => {
+                        assert!(
+                            arg.kind == KernelArgType::MemGlobal
+                                || arg.kind == KernelArgType::MemConstant
+                        );
+                        input.extend_from_slice(null_ptr);
                     }
-                    variable_local_size += *size as u64;
-                }
-                KernelArgValue::Sampler(sampler) => {
-                    samplers.push(sampler.pipe());
-                }
-                KernelArgValue::None => {
-                    assert!(
-                        arg.kind == KernelArgType::MemGlobal
-                            || arg.kind == KernelArgType::MemConstant
-                    );
-                    input.extend_from_slice(null_ptr);
                 }
             }
-        }
 
-        // subtract the shader local_size as we only request something on top of that.
-        variable_local_size -= static_local_size;
+            // subtract the shader local_size as we only request something on top of that.
+            variable_local_size -= static_local_size;
 
-        let mut printf_buf = None;
-        for arg in &self.kernel_info.internal_args {
-            if arg.offset > input.len() {
-                input.resize(arg.offset, 0);
+            let mut printf_buf = None;
+            if nir_kernel_build.printf_info.is_some() {
+                let buf = q
+                    .device
+                    .screen
+                    .resource_create_buffer(printf_size, ResourceType::Staging, PIPE_BIND_GLOBAL)
+                    .unwrap();
+
+                let init_data: [u8; 1] = [4];
+                ctx.buffer_subdata(&buf, 0, init_data.as_ptr().cast(), init_data.len() as u32);
+
+                printf_buf = Some(buf);
             }
-            match arg.kind {
-                InternalKernelArgType::ConstantBuffer => {
-                    assert!(nir_kernel_build.constant_buffer.is_some());
-                    input.extend_from_slice(null_ptr);
-                    resource_info.push((
-                        nir_kernel_build.constant_buffer.clone().unwrap(),
-                        arg.offset,
-                    ));
+
+            for arg in &kernel_info.internal_args {
+                if arg.offset > input.len() {
+                    input.resize(arg.offset, 0);
                 }
-                InternalKernelArgType::GlobalWorkOffsets => {
-                    if q.device.address_bits() == 64 {
-                        input.extend_from_slice(unsafe { as_byte_slice(&offsets) });
-                    } else {
+                match arg.kind {
+                    InternalKernelArgType::ConstantBuffer => {
+                        assert!(nir_kernel_build.constant_buffer.is_some());
+                        let res = nir_kernel_build.constant_buffer.as_ref().unwrap();
+                        add_global(q, &mut input, &mut resource_info, res, 0);
+                    }
+                    InternalKernelArgType::GlobalWorkOffsets => {
+                        if q.device.address_bits() == 64 {
+                            input.extend_from_slice(unsafe {
+                                as_byte_slice(&[
+                                    offsets[0] as u64,
+                                    offsets[1] as u64,
+                                    offsets[2] as u64,
+                                ])
+                            });
+                        } else {
+                            input.extend_from_slice(unsafe {
+                                as_byte_slice(&[
+                                    offsets[0] as u32,
+                                    offsets[1] as u32,
+                                    offsets[2] as u32,
+                                ])
+                            });
+                        }
+                    }
+                    InternalKernelArgType::WorkGroupOffsets => {
+                        workgroup_id_offset_loc = Some(input.len());
+                        input.extend_from_slice(null_ptr_v3);
+                    }
+                    InternalKernelArgType::PrintfBuffer => {
+                        let res = printf_buf.as_ref().unwrap();
+                        add_global(q, &mut input, &mut resource_info, res, 0);
+                    }
+                    InternalKernelArgType::InlineSampler(cl) => {
+                        samplers.push(Sampler::cl_to_pipe(cl));
+                    }
+                    InternalKernelArgType::FormatArray => {
+                        input.extend_from_slice(unsafe { as_byte_slice(&tex_formats) });
+                        input.extend_from_slice(unsafe { as_byte_slice(&img_formats) });
+                    }
+                    InternalKernelArgType::OrderArray => {
+                        input.extend_from_slice(unsafe { as_byte_slice(&tex_orders) });
+                        input.extend_from_slice(unsafe { as_byte_slice(&img_orders) });
+                    }
+                    InternalKernelArgType::WorkDim => {
+                        input.extend_from_slice(&[work_dim as u8; 1]);
+                    }
+                    InternalKernelArgType::NumWorkgroups => {
                         input.extend_from_slice(unsafe {
-                            as_byte_slice(&[
-                                offsets[0] as u32,
-                                offsets[1] as u32,
-                                offsets[2] as u32,
-                            ])
+                            as_byte_slice(&[grid[0] as u32, grid[1] as u32, grid[2] as u32])
                         });
                     }
                 }
-                InternalKernelArgType::PrintfBuffer => {
-                    let buf = Arc::new(
-                        q.device
-                            .screen
-                            .resource_create_buffer(
-                                printf_size,
-                                ResourceType::Staging,
-                                PIPE_BIND_GLOBAL,
-                            )
-                            .unwrap(),
-                    );
-
-                    input.extend_from_slice(null_ptr);
-                    resource_info.push((buf.clone(), arg.offset));
-
-                    printf_buf = Some(buf);
-                }
-                InternalKernelArgType::InlineSampler(cl) => {
-                    samplers.push(Sampler::cl_to_pipe(cl));
-                }
-                InternalKernelArgType::FormatArray => {
-                    input.extend_from_slice(unsafe { as_byte_slice(&tex_formats) });
-                    input.extend_from_slice(unsafe { as_byte_slice(&img_formats) });
-                }
-                InternalKernelArgType::OrderArray => {
-                    input.extend_from_slice(unsafe { as_byte_slice(&tex_orders) });
-                    input.extend_from_slice(unsafe { as_byte_slice(&img_orders) });
-                }
-                InternalKernelArgType::WorkDim => {
-                    input.extend_from_slice(&[work_dim as u8; 1]);
-                }
             }
-        }
-
-        Ok(Box::new(move |q, ctx| {
-            let mut input = input.clone();
-            let mut resources = Vec::with_capacity(resource_info.len());
-            let mut globals: Vec<*mut u32> = Vec::new();
-            let printf_format = &nir_kernel_build.printf_info;
 
             let mut sviews: Vec<_> = sviews
                 .iter()
@@ -1102,19 +1173,11 @@ impl Kernel {
                 .map(|s| ctx.create_sampler_state(s))
                 .collect();
 
-            for (res, offset) in &resource_info {
+            let mut resources = Vec::with_capacity(resource_info.len());
+            let mut globals: Vec<*mut u32> = Vec::with_capacity(resource_info.len());
+            for (res, offset) in resource_info {
                 resources.push(res);
-                globals.push(unsafe { input.as_mut_ptr().add(*offset) }.cast());
-            }
-
-            if let Some(printf_buf) = &printf_buf {
-                let init_data: [u8; 1] = [4];
-                ctx.buffer_subdata(
-                    printf_buf,
-                    0,
-                    init_data.as_ptr().cast(),
-                    init_data.len() as u32,
-                );
+                globals.push(unsafe { input.as_mut_ptr().add(offset) }.cast());
             }
 
             let temp_cso;
@@ -1131,9 +1194,49 @@ impl Kernel {
             ctx.set_sampler_views(&mut sviews);
             ctx.set_shader_images(&iviews);
             ctx.set_global_binding(resources.as_slice(), &mut globals);
-            ctx.update_cb0(&input)?;
 
-            ctx.launch_grid(work_dim, block, grid, variable_local_size as u32);
+            let hw_max_grid: Vec<usize> = q
+                .device
+                .max_grid_size()
+                .into_iter()
+                .map(|val| val.try_into().unwrap_or(usize::MAX))
+                // clamped as pipe_launch_grid::grid is only u32
+                .map(|val| cmp::min(val, u32::MAX as usize))
+                .collect();
+
+            for z in 0..div_round_up(grid[2], hw_max_grid[2]) {
+                for y in 0..div_round_up(grid[1], hw_max_grid[1]) {
+                    for x in 0..div_round_up(grid[0], hw_max_grid[0]) {
+                        if let Some(workgroup_id_offset_loc) = workgroup_id_offset_loc {
+                            let this_offsets =
+                                [x * hw_max_grid[0], y * hw_max_grid[1], z * hw_max_grid[2]];
+
+                            if q.device.address_bits() == 64 {
+                                let val = this_offsets.map(|v| v as u64);
+                                input[workgroup_id_offset_loc..workgroup_id_offset_loc + 24]
+                                    .copy_from_slice(unsafe { as_byte_slice(&val) });
+                            } else {
+                                let val = this_offsets.map(|v| v as u32);
+                                input[workgroup_id_offset_loc..workgroup_id_offset_loc + 12]
+                                    .copy_from_slice(unsafe { as_byte_slice(&val) });
+                            }
+                        }
+
+                        let this_grid = [
+                            cmp::min(hw_max_grid[0], grid[0] - hw_max_grid[0] * x) as u32,
+                            cmp::min(hw_max_grid[1], grid[1] - hw_max_grid[1] * y) as u32,
+                            cmp::min(hw_max_grid[2], grid[2] - hw_max_grid[2] * z) as u32,
+                        ];
+
+                        ctx.update_cb0(&input)?;
+                        ctx.launch_grid(work_dim, block, this_grid, variable_local_size as u32);
+
+                        if Platform::dbg().sync_every_event {
+                            ctx.flush().wait();
+                        }
+                    }
+                }
+            }
 
             ctx.clear_global_binding(globals.len() as u32);
             ctx.clear_shader_images(iviews.len() as u32);
@@ -1149,22 +1252,15 @@ impl Kernel {
 
             if let Some(printf_buf) = &printf_buf {
                 let tx = ctx
-                    .buffer_map(
-                        printf_buf,
-                        0,
-                        printf_size as i32,
-                        RWFlags::RD,
-                        ResourceMapType::Normal,
-                    )
-                    .ok_or(CL_OUT_OF_RESOURCES)?
-                    .with_ctx(ctx);
+                    .buffer_map(printf_buf, 0, printf_size as i32, RWFlags::RD)
+                    .ok_or(CL_OUT_OF_RESOURCES)?;
                 let mut buf: &[u8] =
                     unsafe { slice::from_raw_parts(tx.ptr().cast(), printf_size as usize) };
                 let length = u32::from_ne_bytes(*extract(&mut buf));
 
                 // update our slice to make sure we don't go out of bounds
                 buf = &buf[0..(length - 4) as usize];
-                if let Some(pf) = printf_format.as_ref() {
+                if let Some(pf) = &nir_kernel_build.printf_info {
                     pf.u_printf(buf)
                 }
             }
