@@ -352,7 +352,11 @@ static VAStatus vlVaPostProcBlit(vlVaDriver *drv, vlVaContext *context,
    if ((src->buffer_format == PIPE_FORMAT_B8G8R8X8_UNORM ||
         src->buffer_format == PIPE_FORMAT_B8G8R8A8_UNORM ||
         src->buffer_format == PIPE_FORMAT_R8G8B8X8_UNORM ||
-        src->buffer_format == PIPE_FORMAT_R8G8B8A8_UNORM) &&
+        src->buffer_format == PIPE_FORMAT_R8G8B8A8_UNORM ||
+        src->buffer_format == PIPE_FORMAT_B10G10R10X2_UNORM ||
+        src->buffer_format == PIPE_FORMAT_B10G10R10A2_UNORM ||
+        src->buffer_format == PIPE_FORMAT_R10G10B10X2_UNORM ||
+        src->buffer_format == PIPE_FORMAT_R10G10B10A2_UNORM) &&
        !src->interlaced)
       grab = true;
 
@@ -519,27 +523,6 @@ vlVaApplyDeint(vlVaDriver *drv, vlVaContext *context,
    return context->deint->video_buffer;
 }
 
-static bool can_convert_with_efc(vlVaSurface *src, vlVaSurface *dst)
-{
-   enum pipe_format src_format, dst_format;
-
-   if (src->buffer->interlaced)
-      return false;
-
-   src_format = src->buffer->buffer_format;
-
-   if (src_format != PIPE_FORMAT_B8G8R8A8_UNORM &&
-       src_format != PIPE_FORMAT_R8G8B8A8_UNORM &&
-       src_format != PIPE_FORMAT_B8G8R8X8_UNORM &&
-       src_format != PIPE_FORMAT_R8G8B8X8_UNORM)
-      return false;
-
-   dst_format = dst->encoder_format != PIPE_FORMAT_NONE ?
-      dst->encoder_format : dst->buffer->buffer_format;
-
-   return dst_format == PIPE_FORMAT_NV12;
-}
-
 VAStatus
 vlVaHandleVAProcPipelineParameterBufferType(vlVaDriver *drv, vlVaContext *context, vlVaBuffer *buf)
 {
@@ -581,35 +564,38 @@ vlVaHandleVAProcPipelineParameterBufferType(vlVaDriver *drv, vlVaContext *contex
    src_region = vlVaRegionDefault(param->surface_region, src_surface, &def_src_region);
    dst_region = vlVaRegionDefault(param->output_region, dst_surface, &def_dst_region);
 
-   if (!param->num_filters &&
+   /* EFC can only do one conversion, and it must be the last postproc
+    * operation immediately before encoding.
+    * Disable EFC completely if this is not the case. */
+   if (drv->last_efc_surface) {
+      vlVaSurface *surf = drv->last_efc_surface;
+      surf->efc_surface = NULL;
+      drv->last_efc_surface = NULL;
+      drv->efc_count = -1;
+   }
+
+   if (drv->efc_count >= 0 && !param->num_filters &&
        src_region->width == dst_region->width &&
        src_region->height == dst_region->height &&
        src_region->x == dst_region->x &&
        src_region->y == dst_region->y &&
-       can_convert_with_efc(src_surface, dst_surface) &&
-       pscreen->get_video_param(pscreen,
-                                PIPE_VIDEO_PROFILE_UNKNOWN,
-                                PIPE_VIDEO_ENTRYPOINT_ENCODE,
-                                PIPE_VIDEO_CAP_EFC_SUPPORTED)) {
+       pscreen->is_video_target_buffer_supported &&
+       pscreen->is_video_target_buffer_supported(pscreen,
+                                                 dst_surface->buffer->buffer_format,
+                                                 src_surface->buffer,
+                                                 PIPE_VIDEO_PROFILE_UNKNOWN,
+                                                 PIPE_VIDEO_ENTRYPOINT_ENCODE)) {
 
-      vlVaSurface *surf = dst_surface;
+      dst_surface->efc_surface = src_surface;
+      drv->last_efc_surface = dst_surface;
 
-      // EFC will convert the buffer to a format the encoder accepts
-      if (src_surface->buffer->buffer_format != surf->buffer->buffer_format) {
-         surf->encoder_format = surf->buffer->buffer_format;
-
-         surf->templat.interlaced = src_surface->templat.interlaced;
-         surf->templat.buffer_format = src_surface->templat.buffer_format;
-         surf->buffer->destroy(surf->buffer);
-
-         if (vlVaHandleSurfaceAllocate(drv, surf, &surf->templat, NULL, 0) != VA_STATUS_SUCCESS)
-            return VA_STATUS_ERROR_ALLOCATION_FAILED;
-      }
-
-      pipe_resource_reference(&(((struct vl_video_buffer *)(surf->buffer))->resources[0]), ((struct vl_video_buffer *)(src_surface->buffer))->resources[0]);
-      context->target = surf->buffer;
-
-      return VA_STATUS_SUCCESS;
+      /* Do the blit for first few conversions as a fallback in case EFC
+       * could not be used (see above), after that assume EFC can always
+       * be used and skip the blit. */
+      if (drv->efc_count < 16)
+         drv->efc_count++;
+      else
+         return VA_STATUS_SUCCESS;
    }
 
    src = src_surface->buffer;
